%% Compose two vector fields
function [vx,vy] = compose(ax,ay,bx,by)

    if isequal(ax,zeros(size(ax))) && isequal(ay,zeros(size(ay)))
        vx = bx;        vy = by;        return;
    end;

    % Piggyback
    piggybackscale = 1.2; % just to get too many outside values
    [ax,lim] = piggyback(ax,piggybackscale);
    [ay,lim] = piggyback(ay,piggybackscale);
    [bx,lim] = piggyback(bx,piggybackscale);
    [by,lim] = piggyback(by,piggybackscale);

    % Coordinates
    nx = size(ax,2);
    ny = size(ax,1);
    
    [y,x] = ndgrid(1:ny, 1:nx); % coordinate image

    % Where points are going
    xp  = iminterpolate(x+ax, bx,by);
    yp  = iminterpolate(y+ay, bx,by);
    
    % Update field
    vx = xp - x;
    vy = yp - y;
    
    % Zero vectors going outside the image
    zr  = (xp==0 & yp==0);
    vx(zr) = 0;
    vy(zr) = 0;

    % Unpiggyback
    vx = vx(lim(1):lim(2),lim(3):lim(4));
    vy = vy(lim(1):lim(2),lim(3):lim(4));
    
end

