% equalizehistogram
% Created: Jan. 10th, 2012 - Changed: Jan. 10th, 2012
%
% Equalize histogram of intensities
%
% nbins - number of bins to use in histogram (intensity resolution)
% idx   - in which pixels to compute histogram (e.g., could remove zeros,
%         idx=(I~=0) )
%
function I = equalizehistogram(I,nbins,idx)

    if nargin<2; nbins = 1000;       end;                    % number of bins to use in histogram
    if nargin<3; idx   = 1:numel(I); end;                    % where to compute the histogram
    
    % Get cumulative histogram
    im = I(idx);
    [pdf,x] = hist(im,nbins);                                % histogram
    cdf = zeros(1,size(pdf,2));
    for i=1:nbins; cdf(i) = sum(pdf(1:i)); end;              % cumulative histogram
    
    % Change intensities
    imn = (im-min(im(:)))/range(im(:));                      % normalized intensities (0..1)
    %imi = floor(imn*(nbins-1))+1;                            % integer (index in cdf)
    n   = sum(pdf);                                          % number of pixels
    %ime = cdf(imi)/n;                                       % equalized intensities
    ime = interp1(1:nbins,cdf,imn*(nbins-1)+1,'linear')'/n;  % interpolate cdf for all normalized intensities
    ime = ime*range(im(:)) + min(im(:));                     % change range
    
    % return image
    %I(I~=0) = ime;
    I(idx) = ime;

end