%% Find update between two images
%  Created: Dec 16th, 2011 - Changed: Dec 16th 2011
function [ux,uy] = findneighboringcorrespondence(F,M,Ef,Em)

    %[ux,uy] = findneighboringcorrespondence2(F,M,Ef,Em); return;
    
    addpath('../asclepios'); % knn
    
    % index in masks
%     Fnz = zeros(size(Ef,1),size(Ef,2));
%     Mnz = zeros(size(Em,1),size(Em,2));
%     for i=1:size(Ef,3)
%         Fnz = Fnz | Ef(:,:,i)~=0; % Non zero points
%         Mnz = Mnz | Em(:,:,i)~=0;
%     end
    Fnz  = (F>0);
    Mnz  = (M>0);
    Fidx = find(Fnz);
    Midx = find(Mnz);
    Fallidx = 1:numel(F);
    Mallidx = 1:numel(M);
    
    % Spectral coordinates
    ne = size(Ef,3);
    Ef = reshape(Ef,numel(F),ne);
    Em = reshape(Em,numel(M),ne);
    
    % Add neighbors as channels (enforce similar neighborhood in space)
    if 0
        w = 0.15; % weight of neighbors;
        
        Eff = zeros(numel(F),5*ne);
        Emm = zeros(numel(M),5*ne);
        
        % Add neighbors to its left
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i-1 + (i==1),j);
        Eff(Fallidx(Fidx),1:(2*ne)) = [Ef(Fallidx(Fidx),:) w*Ef(idx,:)];

        [i,j] = ind2sub(size(F),Mallidx(Midx));
        idx = sub2ind(size(F),i-1 + (i==1),j);
        Emm(Mallidx(Midx),1:(2*ne)) = [Em(Mallidx(Midx),:) w*Em(idx,:)];

        % Add neighbors to its right
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i+1 - (i==size(F,1)),j);
        Eff(Fallidx(Fidx),(1+2*ne):(3*ne)) = [w*Ef(idx,:)];

        [i,j] = ind2sub(size(F),Mallidx(Midx));
        idx = sub2ind(size(F),i+1 - (i==size(F,1)),j);
        Emm(Mallidx(Midx),(1+2*ne):(3*ne)) = [w*Em(idx,:)];

        % Add neighbors to its bottom
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i, j-1 + (j==1));
        Eff(Fallidx(Fidx),(1+3*ne):(4*ne)) = [w*Ef(idx,:)];

        [i,j] = ind2sub(size(F),Mallidx(Midx));
        idx = sub2ind(size(F),i, j-1 + (j==1));
        Emm(Mallidx(Midx),(1+3*ne):(4*ne)) = [w*Em(idx,:)];

        % Add neighbors to its top
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i, j+1 - (j==size(F,2)));
        Eff(Fallidx(Fidx),(1+4*ne):(5*ne)) = [w*Ef(idx,:)];

        [i,j] = ind2sub(size(F),Mallidx(Midx));
        idx = sub2ind(size(F),i, j+1 - (j==size(F,2)));
        Emm(Mallidx(Midx),(1+4*ne):(5*ne)) = [w*Em(idx,:)];
        
        Ef = Eff;
        Em = Emm;
    end

    % Add neighbors as average (enforce similar neighborhood in space, invariant to rotation)
    if 0
        % Add neighbors to its left
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i-1 + (i==1),j);
        Ef = [Ef+Ef(idx,:)];
        Em = [Em+Em(idx,:)];

        % Add neighbors to its right
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i+1 - (i==size(F,1)),j);
        Ef = [Ef+Ef(idx,:)];
        Em = [Em+Em(idx,:)];

        % Add neighbors to its bottom
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i, j-1 + (j==1));
        Ef = [Ef+Ef(idx,:)];
        Em = [Em+Em(idx,:)];

        % Add neighbors to its top
        [i,j] = ind2sub(size(F),Fallidx(Fidx));
        idx = sub2ind(size(F),i, j+1 - (j==size(F,2)));
        Ef = [Ef+Ef(idx,:)];
        Em = [Em+Em(idx,:)];
        
        % Average
        Ef = Ef /4;
        Em = Em /4;
    end

    % Find closest spectral point
    tree = kdtree(Em(Midx,:));
    corr = kdtree_closestpoint(tree,Ef(Fidx,:)); % Ef(Fidx(i)) -> Em(Midx(corr(i)))
    
%     % Find closest spectral point from M to F
%     tree_inv = kdtree(Ef(Fidx,:));
%     corr_inv = kdtree_closestpoint(tree_inv,Em(Midx,:)); % Em(Midx(i)) -> Ef(Fidx(corr_inv(i)))
    
    %%%%%%
    % Correspondence between mesh X and Y (F and M)
    %
    %        x_{i}                    y_{corr(i)}
    %         X1  o ---------------> o Y1
    %             |                  |
    %             v                  | neighbor in Y
    % x_{i+1} X2  o                  v
    %         X2' o <--------------- o Y2
    % x_{corr_inv(corr(i)+1)}         y({corr(i)+1}
    %
    % We want minimal difference between neighbors:
    %
    %   |X1-X2'| - |Y1-Y2|, or
    %   |X2' - X2|
    %
    
%     % Index on Y of neighbor on Y
%     idxX1  = Fidx;%(1:numel(F))';
%     [i,j]  = ind2sub(size(F),idxX1);
%     idxX2  = sub2ind(size(F),i+1 - (i==size(F,1)),j);
%     
%     % Index of direct closest points
%     idxY1  = Midx(corr(1:numel(idxX1)));
%     
%     % Index on Y of neighbor on Y
%     [i,j]  = ind2sub(size(F),idxY1);
%     idxY2  = sub2ind(size(F),i+1 - (i==size(F,1)),j);
%     
%     % Index on X of inverse of neighbor in Y
%     idxX2p = corr_inv(idxY2);
%     
%     % Distance
%     d = sum((Ef(idxX2p,:)-Ef(idxX2,:)).^2,2);
%     
%     if 0
%         % Show distance from X2 to X2p
%         imagesc(reshape(sqrt(d),100,100)); % neighbors distance in embedding
%     end
%     
%     if 0
%         % Show distance from X2 to X2p
%         [y,x] = ndgrid(1:100,1:100);
%         a = (x(idxX2p)-x(idxX2)).^2 + (y(idxX2p)-y(idxX2)).^2;
%         imagesc(reshape(sqrt(a),100,100)); % neighbors distance in space
%     end
%     
%     if 0
%         % Show distance from X1 to Y1
%         [y,x] = ndgrid(1:100,1:100);
%         a = (x(idxX1)-x(idxY1)).^2 + (y(idxX1)-y(idxY1)).^2;
%         imagesc(reshape(sqrt(a),100,100)); % distance between corresponding point in space
%     end
    
    
    % Find vector field
    [y,x] = ndgrid(1:size(F,2), 1:size(F,1)); % coordinate image
    ux = zeros(size(F)); uy = zeros(size(F));
    ux(Fallidx(Fidx)) = ( x(Mallidx(Midx(corr))) - x(Fallidx(Fidx)) );
    uy(Fallidx(Fidx)) = ( y(Mallidx(Midx(corr))) - y(Fallidx(Fidx)) );
    
end




function [ux,uy] = findneighboringcorrespondence2(F,M,Ef,Em)

    addpath('../asclepios'); % knn
    
    % index in masks
    Fnz = zeros(size(Ef,1),size(Ef,2));
    Mnz = zeros(size(Em,1),size(Em,2));
    for i=1:size(Ef,3)
        Fnz = Fnz | Ef(:,:,i)~=0; % Non zero points
        Mnz = Mnz | Em(:,:,i)~=0;
    end
    Fidx = find(Fnz);
    Midx = find(Mnz);
    
    % Spectral coordinates
    Ef = reshape(Ef,numel(F),size(Ef,3));
    Em = reshape(Em,numel(M),size(Em,3));
    
    % Find closest spectral point
    tree = kdtree(Em(Midx,:));
    corr = kdtree_closestpoint(tree,Ef(Fidx,:)); % Ef(Fidx(i)) -> Em(Midx(corr(i)))
    
    % Find closest spectral point from M to F
    tree_inv = kdtree(Ef(Fidx,:));
    corr_inv = kdtree_closestpoint(tree_inv,Em(Midx,:)); % Em(Midx(i)) -> Ef(Fidx(corr_inv(i)))
    
    %%%%%%
    % Correspondence between mesh X and Y (F and M)
    %
    %        x_{i}                    y_{corr(i)}
    %         X1  o ---------------> o Y1
    %             |                  |
    %             v                  | neighbor in Y
    % x_{i+1} X2  o                  v
    %         X2' o <--------------- o Y2
    % x_{corr_inv(corr(i)+1)}         y({corr(i)+1}
    %
    % We want minimal difference between neighbors:
    %
    %   |X1-X2'| - |Y1-Y2|, or
    %   |X2' - X2|
    %
    
    % Index on Y of neighbor on Y
    idxX1  = (1:numel(F))';
    [i,j]  = ind2sub(size(F),idxX1);
    idxX2  = sub2ind(size(F),i+1 - (i==size(F,1)),j);
    
    % Index of direct closest points
    idxY1  = corr(idxX1);
    
    % Index on Y of neighbor on Y
    [i,j]  = ind2sub(size(F),idxY1);
    idxY2  = sub2ind(size(F),i+1 - (i==size(F,1)),j);
    
    % Index on X of inverse of neighbor in Y
    idxX2p = corr_inv(idxY2);
    
    % Distance
    d = sum((Ef(idxX2p,:)-Ef(idxX2,:)).^2,2);
    
    if 1
        % Show distance from X2 to X2p
        imagesc(reshape(sqrt(d),100,100)); % neighbors distance in embedding
    end
    
    if 1
        % Show distance from X2 to X2p
        [y,x] = ndgrid(1:100,1:100);
        a = (x(idxX2p)-x(idxX2)).^2 + (y(idxX2p)-y(idxX2)).^2;
        imagesc(reshape(sqrt(a),100,100)); % neighbors distance in space
    end
    
    if 1
        % Show distance from X1 to Y1
        [y,x] = ndgrid(1:100,1:100);
        a = (x(idxX1)-x(idxY1)).^2 + (y(idxX1)-y(idxY1)).^2;
        imagesc(reshape(sqrt(a),100,100)); % distance between corresponding point in space
    end
    
    % Find vector field
    [y,x] = ndgrid(1:size(F,2), 1:size(F,1)); % coordinate image
    ux = zeros(size(F)); uy = zeros(size(F));
    ux(Fidx) = ( x(Midx(corr)) - x(Fidx) );
    uy(Fidx) = ( y(Midx(corr)) - y(Fidx) );
    
end

