%% scale image with another
%  Scale mass of Y with X
%  Created: Jan 3rd, 2011
%
function [Y,T,sx,sy] = findrigid(X,Y,mode)

    if nargin<3; mode = 'nearest'; end;

    % points in mask
    th = 0.1;
    idxX = find(X>th);
    idxY = find(Y>th);
    nX  = size(idxX,1);
    nY  = size(idxY,1);
    
    % coordinates of points
    [Xsub(:,1) Xsub(:,2)] = ind2sub(size(X),idxX); % coordinates of points in mask
    [Ysub(:,1) Ysub(:,2)] = ind2sub(size(Y),idxY);
    
    % center points
    Xc = mean(Xsub); % center of mass
    Yc = mean(Ysub);
    
    Xp = bsxfun(@minus,Xsub,Xc); % center points
    Yp = bsxfun(@minus,Ysub,Yc);
    
    % Build covariance matrix
    Hx = Xp'*Xp / nX;  % covariance matrix
    Hy = Yp'*Yp / nY;
    
    % Get principal axis
    [ux,vx] = eig(Hx); % pca
    [uy,vy] = eig(Hy);
    
    % Get scaling factor along principal axis
    scale = sqrt( diag(vy) ./ diag(vx) ); % scale factor
    scale(diag(vx)==0) = 1;
    scale = diag(scale);
    scale(3,3) = 1;

    % Get rotation (and scaling) from X to Y
    Rx = ux;    Rx(3,3) = 1;
    Ry = uy;    Ry(3,3) = 1;
    R  = Ry * scale * Rx;
    
    % Translate to origin before rotating
    t = eye(3); t(3,1:2) = Xc;
    T = (t)^-1 * R * (t);
    
    % applying transformation
    [Y,sx,sy] = imaffine(Y,T,mode);
    
end

