%% Find update between two images
%  Changed: Dec 7th, 2011
%
function [ux,uy] = findupdate(F,M,sx,sy,sigma_i,sigma_x,mode)

    if nargin<7; mode = 'linear'; end;
    
    % check if multichannel
    if size(size(F),2)==3; [ux,uy] = findupdate_multichannel(F,M,sx,sy,sigma_i,sigma_x,mode); return; end;
        
    % Interpolate updated image
    M_prime = iminterpolate(M,sx,sy,mode); % intensities at updated points
    area    = size(M,1)*size(M,2);    % area of moving image
    
    % image difference
    diff = F - M_prime;
    
    % fixed image gradient
    [gx_f,gy_f] = gradient(F);     % image gradient
    normg2_f  = gx_f.^2 + gy_f.^2; % squared norm of gradient
    
    % moving image gradient
    [gx,gy] = gradient(M_prime);   % image gradient
    normg2  = gx.^2 + gy.^2;       % squared norm of gradient
    
    % update is Idiff / (||J||^2+(Idiff^2)/sigma_x^2) J, with Idiff = F(x)-M(x+s), and J = Grad(M(x+s));
    scale = diff ./ (normg2 + diff.^2*sigma_i^2/sigma_x^2);
    scale(normg2==0) = 0;
    scale(diff  ==0) = 0;
    %scale = scale .* ((scale>=0) + (scale<0) .* sign(gx.*gx_f + gy.*gy_f)); % avoid collapsing gradients (change sign if moving goes backward)
    ux = gx .* scale;
    uy = gy .* scale;
    
    % Zero non overlapping areas
    %ux(F==0)       = 0; uy(F==0)       = 0;
    %ux(M_prime==0) = 0; uy(M_prime==0) = 0;

end

%% Find update between two images
%  Changed: Dec 7th, 2011
%
function [ux,uy] = findupdate_multichannel(F,M,sx,sy,sigma_i,sigma_x,alpha,mode)

    if nargin<7; mode = 'linear'; end;
    
    % number of channels
    nchannels = size(F,3);
    
    % Channels weights
    if nargin<7; alpha = 1; end;
    if numel(alpha)==1; alpha = ones(1,nchannels)*alpha/nchannels; end;
    
    % Average update of all subchannels
    ux = zeros(size(sx));
    uy = zeros(size(sy));
    for i=1:nchannels
        
        [ux_c,uy_c] = findupdate(F(:,:,i),M(:,:,i),sx,sy,sigma_i,sigma_x,mode);
        
        ux = ux + alpha(i)*ux_c;
        uy = uy + alpha(i)*uy_c;
        
    end
    
end

