%% Random deformation
function [I,sx,sy,sx_inv,sy_inv,ctrl_pts,delta,vx,vy] = randomdeform(I,npts,maxdeform,sigma,seed_rnd)

    if nargin<2; npts      = 5; end;               % max number of control points
    if nargin<3; maxdeform = max(size(I))/10; end; % max 1/10 of image width for deformation
    if nargin<4; sigma     = max(size(I))/2;  end; % max smooth on whole image (radius of each control points)
    if nargin<5; seed_rnd  = 1;               end; % seed for random number generator

    % Create random deformation
    s  = RandStream.create('mt19937ar','seed',seed_rnd); RandStream.setDefaultStream(s); % always same random numbers
    ctrl_pts = floor(bsxfun(@times,rand(npts,2),size(I)-1)) + 1; % position of control points
    delta    = 2*(rand(npts,2)-.5) * maxdeform * 2*pi*sigma^2;    % displacement of ctrl points between -maxdeform..+maxdeform
    
    sx = zeros(size(I));
    sy = zeros(size(I));
    
    sx(sub2ind(size(I),ctrl_pts(:,1),ctrl_pts(:,2))) = delta(:,2);
    sy(sub2ind(size(I),ctrl_pts(:,1),ctrl_pts(:,2))) = delta(:,1);
    
    delta = delta / (2*pi*sigma^2);
    
    %sx = maxdeform * 2*(rand(size(I))-0.5);
    %sy = maxdeform * 2*(rand(size(I))-0.5);

    % Smooth deformation
    sx = imgaussian(sx,sigma);
    sy = imgaussian(sy,sigma);
    
    vx = sx; vy = sy;
    [sx,sy] = expfield(vx,vy);
    [sx_inv,sy_inv] = expfield(-vx,-vy);

    % Interpolate updated image
    I = iminterpolate(I,sx,sy);

end
