%% Flip signs in embedding components
%  Created: Dec 21st, 2011
%  Changed: Dec 25th, 2011
%
function [B2,V2] = reorderembedding(B1,B2,V1,V2,I1,I2,lambda)

    nchannels = size(B1,3);
    npixels   = size(B1,1)*size(B1,2);
    
    nbins  = 8; % number of bins in joint histograms
    if nargin<7; lambda = 0.5; end; % between 0..1 where 0: total joint histogram, 1: total eigenmode difference
    
    B1s = B1;
    B2s = B2;
    
    %% Normalize eigenvectors between -1..+1
    for i=1:nchannels
        b1 = B1s(:,:,i);
        b2 = B2s(:,:,i);

        %B1s(:,:,i) = 2*(b1-min(b1(:)))/range(b1(:))-1;
        %B2s(:,:,i) = 2*(b2-min(b2(:)))/range(b2(:))-1;
        
        b1(b1<-1) = -1; b1(b1>1) = 1;
        b2(b2<-1) = -1; b2(b2>1) = 1;
        
        B1s(:,:,i) = b1;
        B2s(:,:,i) = b2;
    end
    
    % Find joint histogram
    
    %% Build similarity matrix
    C  = zeros(nchannels,nchannels);
    Cm = zeros(nchannels,nchannels);
    for i=1:nchannels
        for j=1:nchannels
            
            bi = B1s(:,:,i)/2;
            bj = B2s(:,:,j)/2;
            
            % eigenmode and image within mask
            I1_nz = I1(I1~=0);
            I2_nz = I2(I2~=0);
            bi_nz = bi(I1~=0);
            bj_nz = bj(I2~=0);
            npixels2_nz = numel(I1_nz)*numel(I2_nz);
            
            % diff of eigenmode values
            jh  = sqrt( sum(sum((jointhist(I1_nz,bi_nz+.5,nbins) - jointhist(I2_nz,+bj_nz+.5,nbins)).^2)) / npixels2_nz );
            jhm = sqrt( sum(sum((jointhist(I1_nz,bi_nz+.5,nbins) - jointhist(I2_nz,-bj_nz+.5,nbins)).^2)) / npixels2_nz ); % negative eigenvector
            
            % diff of eigenmode values
            diff  = sqrt( sum((bi(:)-bj(:)).^2) / npixels);
            diffm = sqrt( sum((bi(:)+bj(:)).^2) / npixels); % negative eigenvector
            
            % similarity matrix
            C(i,j)  = lambda*diff   + (1-lambda) * jh;
            Cm(i,j) = lambda*diffm  + (1-lambda) * jhm; % negative eigenvector
            
        end
    end
    
    % Build signed similarity matrix
    % Remember where are the sign flips (S:1 if sign flip between E1(:,i) and E2(:,j))
    Q = min(C,Cm);   % optimize for best cost
    S = C > Cm;      % same sign: 0, flip sign: 1
    [rows_flipped,cols_flipped] = find(S); % where are the flips

    %% Find best matching
    M = hungarian(Q);
    [rows,cols] = find(M);

    % Get the permutation costs
    Q = sum(M'*Q')'; % cost for ith pair of eigenvectors

    %% Flip signs
    flipped = intersect([rows,cols],[rows_flipped,cols_flipped],'rows'); % which eigenmaps to flip
    if ~isempty(flipped)
        B2(:,:,flipped(:,2)) = -B2(:,:,flipped(:,2));
    end

    %% Reorder eigenvectors
    B2(:,:,rows) = B2(:,:,cols);
    V2(rows) = V2(cols);
    
end

