%% Framework for testing demons algorithm
%  Created: Dec. 11th, 2011
%  Changed: Dec. 25th, 2011
%
% Runme script that register two images using different flavors of the
% demons algorithm
%
% *** Work in progress ***
%
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
% 
% References to cite:
%
% [1] Spectral Demons - Image Registration via Global Spectral Correspondence, ECCV'12
%     Herve Lombaert, Leo Grady, Xavier Pennec, Nicholas Ayache, Farida Cheriet.
%
function runme

clf;

filenames = { 'lenag2', 'JHU-ch06', 'baseball' };
for filename_iter = 1 %1:3

%% Parameters for intensity registration
niter_spectral  = 50;
niter           = 50;
sigma_fluid     = 1.0; % regularize update      field
sigma_diffusion = 1.1; % regularize deformation field
sigma_i         = 1.0; % weight on similarity term
sigma_x         = 1.0; % weight on spatial uncertainties (maximal step)
diffeomorphic   = 1;   % use exp(u)
nlevel          = 4;   % multiresolution
downsample_spec = 1;   % spectrum on downsample images
descend         = 0;   % break if energy stops descending
do_display      = 1;   % display iterations
do_print        = 0;   % display iterations
do_avi          = 0;   % do animation
do_plotenergy   = 0;   % plot energy

%% Embeddings parameters
ne              = 3;   % number of eigenvectors (minimum 2)  (use 4 for JHU, 2 for RIO, 3 for Lena)
lambda_edges    = 0.99;% weight on graph edges (0..inf)
lambda_nodes    = 0;   % weight on graph nodes (0..1)

%% Registration strategy
use_alignment   = 0;   % use spectral alignment
use_spectral    = 1;   % use spectral correspondence         %%% <----- ENABLE SPECTRAL DEMONS
use_intensity   = 1;   % use intensity registration

%% Parameters for spectral correspondence
nw              = 0;              % null space                (body remains in body)
ew              = 1;              % spectral coordinates      (geometry similarity)
sw              = 2 * (ne-1);     % spatial coordinates       (points remain close)
iw              = 5 * (ne-1+2);   % intensities               (same intensity remains the same)
add_back_spec   = 0;              % add background spectrum

%% Load fixed image
id_fixed  = char(filenames(filename_iter));
id_moving = [id_fixed 'Warped'];
filename  = ['data/' id_fixed '.png'];
F = double(imread(filename));
F = F(1:2:end,1:2:end)+1;         % downsample
F = 1*(F-min(F(:)))/range(F(:));  % normalize intensities

%% Load moving image
s  = RandStream.create('mt19937ar','seed',1); RandStream.setDefaultStream(s); % always same random numbers

%% Restructure heart (labeled image)
if filename_iter == 2 % jhu heart
    sigma_diffusion = 1.2;
    ew              = 1;          % spectral coordinates      (geometry similarity)
    sw              = 9;          % spatial coordinates       (points remain close)
    iw              = 40;         % intensities               (same intensity remains the same)
        
    blood      = 0.5;
    myocardium = 0.9;
    F = myocardium*(F==1) + blood*(F==66/255) + blood*(F==132/255);
    M = myocardium*(M==1) + blood*(M==66/255) + blood*(M==132/255);
end


%% Piggyback image
piggyback_scale = 1.4;
if filename_iter == 2; piggyback_scale = 1.2; end;
[F,lim] = piggyback(F,piggyback_scale);

switch filename_iter
    case 1
    % Deform image (Lena)
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv,ctrl_pts,delta] = randomdeform(F,15,20,10);
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv] = deform(F,[ctrl_pts; 120 100],[delta; 0 30]*1.0,10);
        
    case 2
    % Deform image (Heart JHU)
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv,ctrl_pts,delta] = randomdeform(F,6,15,10);
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv] = deform(F,[ctrl_pts;55 40; 45 40; 45 20; 60 70],[delta; 5 -5; 10 -10; 10 -10; 10 10]*0.9,9);
    
    case 3
    % Deform image (Lena)
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv,ctrl_pts,delta] = randomdeform(F,25,15,10, 2);
    [M,sx_deform,sy_deform,sx_deform_inv,sy_deform_inv] = deform(F,[ctrl_pts],[delta]*1.0,10);
        
end

if do_display
    imagesc(.2*F+M); daspect([1 1 1]); axis off;
end

% Title
addtitle('Registering Moving to Fixed');

% Display initial images
clf;
showimage(F,'Fixed', M,'Moving');

M_original = M;

% Align images using spectrum
if use_alignment
    [M,T,sx_alignment,sy_alignment] = findrigid(F,M,'nearest');
    
    M = iminterpolate(M_original, sx_alignment,sy_alignment,'nearest');
    
    % Title
    addtitle('Rigid Alignment of Eigenmodes');
    
    clf;
    showimage(F,'Fixed', M_original,'Moving', M,'Aligned', F/2+M/2,'Sum');
    drawnow
else
    sx_alignment = zeros(size(F));
    sy_alignment = zeros(size(F));
end
M_aligned = M;
sx_total = sx_alignment;
sy_total = sy_alignment;

% Spectral correspondence
if use_spectral
    
    % Title
    addtitle('Registration using Spectral Correspondence');
    
    sx = zeros(size(F));
    sy = zeros(size(F));
    
    % Scale down images
    if downsample_spec
        scale = 1;%32/size(F,1); % max 16 pixels
        scale = min(scale,1); % always scale down

        Fl  = imresize(F,scale,'nearest');
        Ml  = imresize(M,scale,'nearest');
        sxl = imresize(sx*scale,scale);
        syl = imresize(sy*scale,scale);
        Fl(Fl<0) = 0; Fl(Fl>1) = 1; % valid intensities
        Ml(Ml<0) = 0; Ml(Ml>1) = 1; % valid intensities
    else
        scale = 1;
        Fl = F;
        Ml = M;
    end

    % Options
    opt = struct('niter',niter_spectral,...
                 'sigma_diffusion',sigma_diffusion, ...
                 'min_sigma_corr',sigma_fluid,... %s_sc*scale, ...
                 'ne',ne, ...
                 'lambda_edges',lambda_edges, ...
                 'lambda_nodes',lambda_nodes, ...
                 'null_weight',nw, ...
                 'eigenmode_weight',ew, ...
                 'image_weight',iw, ...
                 'spatial_weight',sw, ...
                 'bkg_weight',add_back_spec, ...
                 'id_fixed',id_fixed,...
                 'id_moving',id_moving,...
                 'do_display',do_display, ...
                 'do_plotenergy',do_plotenergy, ...
                 'do_avi',do_avi, ...
                 'do_print',do_print);
             
    % Conservative vector field (no vortex)
    opt.diffeomorphic = 1;
    [M_spectral,sx_spectral,sy_spectral, sx_spectral_inv,sy_spectral_inv, vx,vy, stats] ...
        = sym_sregister(Fl,Ml,opt);
    
    % Upscale image
    if downsample_spec
        sx_spectral     = imresize(sx_spectral    /scale,size(F));
        sy_spectral     = imresize(sy_spectral    /scale,size(F));
        sx_spectral_inv = imresize(sx_spectral_inv/scale,size(F));
        sy_spectral_inv = imresize(sy_spectral_inv/scale,size(F));
    end

    % Total transformation
    [sx_total,sy_total] = compose(sx_total,sy_total, sx_spectral,sy_spectral);
    
    % Transform image
    M_spectral = iminterpolate(M_original, sx_total,sy_total, 'nearest');
    M_spectral(M_spectral<0) = 0; M_spectral(M_spectral>1) = 1; % valid intensities

    subplot(1,2,2); imagesc(F+M_spectral,[0 2]); daspect([1 1 1]);

    % Freeze moving image
    M = M_spectral;

    % Display difference with ground truth
    diff = sqrt((sx_total-sx_deform_inv).^2 + (sy_total-sy_deform_inv).^2);
    diffmap = diff.*(F>0); diffmap = diffmap/10; % max is 10pixels error
    if do_display
        subplot(2,2,3); imagesc(diffmap); axis equal off; colorbar; caxis([0 10]);
        disp(['RMS mean/std/max: ' num2str([mean(diff(F>0).^2) std(diff(F>0).^2) max(diff(F>0).^2)])]);
        disp(['EUC mean/std/max: ' num2str([mean(diff(F>0)) std(diff(F>0)) max(diff(F>0))])]);
    end

    clf;
    
end

% Image intensity registration
if use_intensity
    
    % Title
    addtitle('Registration using Demons');
    
    vx = zeros(size(F));
    vy = zeros(size(F));
    
    if nlevel == 1

        %% Register
        disp(['Register...']);
        opt = struct('niter',niter, ...
                     'sigma_fluid',sigma_fluid, ...
                     'sigma_diffusion',sigma_diffusion, ...
                     'sigma_i',sigma_i, ...
                     'sigma_x',sigma_x, ...
                     'vx',vx, ...
                     'vy',vy, ...
                     'diffeomorphic',diffeomorphic, ...
                     'descend',descend, ...
                     'id_fixed',id_fixed,...
                     'id_moving',id_moving,...
                     'do_avi',do_avi, ...
                     'do_print',do_print,...
                     'do_display',do_display, ...
                     'do_plotenergy',do_plotenergy);
        [Mp,sx,sy,sx_inv,sy_inv,vx,vy,stats] = sym_register(F,M,opt);

    else

        %% Multiresolution
        for k=nlevel:-1:1
            disp(['Register level: ' num2str(k) '...']);

            % downsample
            mode = 'nearest';
            %mode = 'linear';
            scale = 2^-(k-1);
            Fl  = imresize(F,scale,mode);
            Ml  = imresize(M,scale,mode);
            vxl = imresize(vx*scale,scale);
            vyl = imresize(vy*scale,scale);
            Fl(Fl<0) = 0; Fl(Fl>1) = 1; % valid intensities
            Ml(Ml<0) = 0; Ml(Ml>1) = 1; % valid intensities

            % register
            opt = struct('niter',niter, ...
                         'sigma_fluid',sigma_fluid, ...
                         'sigma_diffusion',sigma_diffusion, ...
                         'sigma_i',sigma_i, ...
                         'sigma_x',sigma_x, ...
                         'vx',vxl, ...
                         'vy',vyl, ...
                         'diffeomorphic',diffeomorphic, ...
                         'descend',descend, ...
                         'do_display',do_display, ...
                         'id_fixed',id_fixed,...
                         'id_moving',id_moving,...
                         'do_avi',do_avi, ...
                         'do_print',do_print,...
                         'do_plotenergy',do_plotenergy);
            [Mp,sxl,syl,sxl_inv,syl_inv,vxl,vyl,stats] = sym_register(Fl,Ml,opt);

            % upsample
            vx     = imresize(vxl/scale,size(F));
            vy     = imresize(vyl/scale,size(F));
            sx     = imresize(sxl/scale,size(F));
            sy     = imresize(syl/scale,size(F));
            sx_inv = imresize(sxl_inv/scale,size(F));
            sy_inv = imresize(syl_inv/scale,size(F));
        end

    end
    
    sx_demons = sx;
    sy_demons = sy;

    % Total transformation
    [sx_total,sy_total] = compose(sx_total,sy_total, sx_demons,sy_demons);
    
    % Get final image
    M_demons = iminterpolate(M_original, sx_total,sy_total, 'nearest');
    M_demons(M_demons<0) = 0; M_demons(M_demons>1) = 1; % valid intensities
    
    M = M_demons;
    
    % Display
    clf;
    if use_spectral
        showimage(F,'Fixed', M_aligned,'Moving + Aligned', M_spectral, '+ Spectral Demons', M,'+ Demons', abs(F-M),'Diff', 'nbrows',2, 'row',1);
    else
        showimage(F,'Fixed', M_aligned,'Moving + Aligned', M_demons,'+ Demons', abs(F-M),'Diff', 'nbrows',2, 'row',1);
    end
    
    % Display Error
    diff = sqrt((sx_total-sx_deform_inv).^2 + (sy_total-sy_deform_inv).^2);
    diffmap = diff.*(F>0); diffmap = diffmap/10; % max is 10pixels error
    if do_display
        subplot(2,2,3); imagesc(diff.*(F>0)); axis equal off; colorbar; caxis([0 10]);
        disp(['RMS mean/std/max: ' num2str([mean(diff(F>0).^2) std(diff(F>0).^2) max(diff(F>0).^2)])]);
        disp(['EUC mean/std/max: ' num2str([mean(diff(F>0)) std(diff(F>0)) max(diff(F>0))])]);
    end
    
end % end intensity


end % for filename_iter
    
end % end function

function addtitle(msg)

    return
    
    global do_avi;
    global aviobj;
    
    clf;
    text(0,1,msg,'fontsize',32); axis off;
    
    if do_avi
        for i=1:10
            aviobj = addframe(aviobj,getframe(gcf));
        end
    end

end
