%% Get spectrum
function [E,V] = spectrum(I,ne,lambda_edge,lambda_node,opt)

    if nargin<2; ne          = 8; end;    % number of eigenvectors to compute
    if nargin<3; lambda_edge = 1; end;    % factor on edge weight (0..10)
    if nargin<4; lambda_node = 1; end;    % factor on node weight (0..1)
    if nargin<5; opt = struct('inv',0); end;

    eps = 1e-6;%0.001; % small epsilon (avoid division by zero)
    
    %% Construct adjacency matrix
    sizex = size(I,1);
    sizey = size(I,2);
    n     = sizex*sizey;

    [x,y] = ndgrid(0:(sizex-1), 0:(sizey-1)); % coordinate image
    
    % Horizontal neighbbors
    idx0x = (x+0) + (y+0)*sizex + 1; idx0x = idx0x(1:(sizex-1),1:sizey);
    idx1x = (x+1) + (y+0)*sizex + 1; idx1x = idx1x(1:(sizex-1),1:sizey);
    
    % Vertical neighbbors
    idx0y = (x+0) + (y+0)*sizex + 1; idx0y = idx0y(1:sizex,1:(sizey-1));
    idx1y = (x+0) + (y+1)*sizex + 1; idx1y = idx1y(1:sizex,1:(sizey-1));

    % Diagonal neighbbors
    idx0d1 = (x+0) + (y+0)*sizex + 1; idx0d1 = idx0d1(1:(sizex-1),1:(sizey-1));
    idx1d1 = (x+1) + (y+1)*sizex + 1; idx1d1 = idx1d1(1:(sizex-1),1:(sizey-1));

    % Diagonal neighbbors
    idx0d2 = (x+0) + (y+0)*sizex + 1; idx0d2 = idx0d2(2:sizex,1:(sizey-1));
    idx1d2 = (x-1) + (y+1)*sizex + 1; idx1d2 = idx1d2(2:sizex,1:(sizey-1));

    % Horizontal second neighbbors
    idx0x2 = [];%(x+0) + (y+0)*sizex + 1; idx0x2 = idx0x2(1:(sizex-2),1:sizey);
    idx1x2 = [];%(x+2) + (y+0)*sizex + 1; idx1x2 = idx1x2(1:(sizex-2),1:sizey);
    
    % Vertical second neighbbors
    idx0y2 = [];%(x+0) + (y+0)*sizex + 1; idx0y2 = idx0y2(1:sizex,1:(sizey-2));
    idx1y2 = [];%(x+0) + (y+2)*sizex + 1; idx1y2 = idx1y2(1:sizex,1:(sizey-2));

    % Construct neighborhood
    F(:,1) = [idx0x(:); idx0y(:); idx0d1(:); idx0d2(:); idx0x2(:); idx0y2(:)];
    F(:,2) = [idx1x(:); idx1y(:); idx1d1(:); idx1d2(:); idx1x2(:); idx1y2(:)];
 
    % Get weights
    dist2    = (x(F(:,1))-x(F(:,2))).^2 + (y(F(:,1))-y(F(:,2))).^2;  % length of each edge
    diff2    = ( I(F(:,1)) - I(F(:,2)) ).^2;                         % intensity difference along edge
    egde_weights  = (lambda_edge*diff2/max(diff2(:)) + dist2/max(dist2(:)) + eps).^-1;
    %egde_weights  = (dist2/max(dist2(:)) + eps).^-1;
    
    W = sparse(F(:,1),F(:,2),egde_weights,n,n);
    W = W + W';
    
    %% Construct Laplacian matrix
    D = spdiags(sum(W)',0,speye(n)); % D = diag(sum(W));
    L = D - W;

    % Node weight
    %node_weights = lambda_node*(1-I(:)) + (1-lambda_node)*ones(n,1) + eps;
    %node_weights = ( lambda_node*(I(:).^2/max(I(:).^2)) + (1-lambda_node)*ones(n,1) + eps );%.^-1;
    %node_weights = node_weights / sum(node_weights);
    %node_weights = node_weights.^-1;
    s = 0.01;
    node_weights = exp(-I(:).^2/(2*s^2));
    %node_weights = 1-I(:)+eps;
    node_weights = lambda_node*(node_weights) + (1-lambda_node)*ones(n,1) + eps;
    if opt.inv; node_weights = 1-node_weights+2*eps; end;
    
    % Normalize
    Dinv = spdiags(sum(W)'.^-1,0,speye(n));  % Dinv = D^-1;
    N    = spdiags(node_weights,0,speye(n)); % Node weight
    G    = Dinv .* N;
    L    = G * L;
    %L    = (G.^0.5) * L * (G.^0.5);

    %% Get Eigenvectors
    opt   = struct('disp',0, 'isreal',1, 'issym',0, 'maxit',50, 'tol',1e-10);
    [E,V,flag] = eigs(L,ne,'sm',opt); % arpack eigen decomposition
    if flag~=0; disp('All eigenvalues did not converge'); end;
    V = diag(V);
    
    % Reorder eigenvalues
    [V,idx] = sort(V,'ascend');
    E = E(:,idx);
    
    % Reshape matrix
    E = reshape(E,sizex,sizey,ne);

end
