%% Get spectrum of image
%  Changed: Dec 11th, 2011
%
function [E,V,idx_check] = spectrum_image(I,ne,lambda_edge,lambda_node,opt)

    if nargin<5; opt = struct('inv',0, 'idx_check',0); end;
    if ~isfield(opt,'inv');       opt.inv       = 0; end;
    if ~isfield(opt,'idx_check'); opt.idx_check = 0; end;

    piggybackscale = 1.5; % piggyback image before getting eigenmodes

    s  = RandStream.create('mt19937ar','seed',1); RandStream.setDefaultStream(s); % always same random numbers
    
    %idx_mask    = find(I>0);
    %idx_notmask = find(I<=0);
    idx_mask = 1:numel(I);

    [I,lim] = piggyback(I,piggybackscale); % move boundary far away

    %[E,V] = spectrum_mask(I,ne,lamba_edge,lamba_node);
    [E,V] = spectrum(I,ne,lambda_edge,lambda_node,opt);

    I = I(lim(1):lim(2),lim(3):lim(4));
    E = E(lim(1):lim(2),lim(3):lim(4),:);

    if 0
        % Check fiedler extrema is closest to origin (flip sign based on its 
        if opt.idx_check<1
            null = E(:,:,1);           % null vector
            center_x = size(I,2)*0.25; % extrama closest to this point is always positive
            center_y = size(I,2)*0.75;
            [unused,idx_min_null] = min(null(idx_mask)); idx_min_null = idx_mask(idx_min_null);
            [unused,idx_max_null] = max(null(idx_mask)); idx_max_null = idx_mask(idx_max_null);
            [y_min x_min] = ind2sub(size(I),idx_min_null); norm_min2 = (x_min-center_x)^2 + (y_min-center_y)^2;
            [y_max x_max] = ind2sub(size(I),idx_max_null); norm_max2 = (x_max-center_x)^2 + (y_max-center_y)^2;
            if norm_min2<norm_max2; idx_check_null = idx_min_null;
            else                    idx_check_null = idx_max_null; end;
            [y,x] = ind2sub(size(I),idx_check_null);
            %disp(['min point_null:    ' num2str([x y])]);

            % Check fiedler extrema is closest to origin (flip sign based on it
            % eigenvector value
            fielder = E(:,:,2);        % fiedler vector
            center_x = size(I,2)*0.25; % extrama closest to this point is always positive
            center_y = size(I,2)*0.75;
            [unused,idx_min_fiedler] = min(fielder(idx_mask)); idx_min_fiedler = idx_mask(idx_min_fiedler);
            [unused,idx_max_fiedler] = max(fielder(idx_mask)); idx_max_fiedler = idx_mask(idx_max_fiedler);
            [y_min x_min] = ind2sub(size(I),idx_min_fiedler); norm_min2 = (x_min-center_x)^2 + (y_min-center_y)^2;
            [y_max x_max] = ind2sub(size(I),idx_max_fiedler); norm_max2 = (x_max-center_x)^2 + (y_max-center_y)^2;
            if norm_min2<norm_max2; idx_check_fiedler = idx_min_fiedler;
            else                    idx_check_fiedler = idx_max_fiedler; end;
            [y,x] = ind2sub(size(I),idx_check_fiedler);
            %disp(['min point_fiedler: ' num2str([x y])]);

            [i1,j1] = ind2sub(size(I),idx_check_null);
            [i2,j2] = ind2sub(size(I),idx_check_fiedler);
            idx_check = sub2ind(size(I),round((i1*0.5+i2*0.5)),round((j1*0.5+j2*0.5)));
            %idx_check = sub2ind(size(I),round((i1*0.33+i2*0.33+0*0.33)),round((j1*0.33+j2*0.33+0*0.33)));
            idx_check = idx_check_fiedler;
        else
            idx_check = opt.idx_check;
        end

        % Flip sign
        for i=1:ne
            Ei = E(:,:,i);
            if Ei(idx_check)<0; E(:,:,i) = -E(:,:,i); end; % first mask pixel in eigenvector
        end
    end
    
    if 1
        % Check if eigenvector signs match
        if isfield(opt,'E')
            for i=1:ne
                e1 = opt.E(:,:,i);
                e2 = E(:,:,i);
                
                e1 = 2*(e1-min(e1(:)))/range(e1(:))-1;
                e2 = 2*(e2-min(e2(:)))/range(e2(:))-1;

                diff1 = sum((e1(:)-e2(:)).^2);
                diff2 = sum((e1(:)+e2(:)).^2);

                if diff2<diff1; E(:,:,i) = -E(:,:,i); end;
            end
        end
        idx_check = 1;
    end

    % Scale eigenvector to fit -1:+1
    for i=1:ne
        Ei      = E(:,:,i);
        Ei_mask = Ei(idx_mask);                  % eigenvector values in mask

        %E(:,:,i) = 2*(E(:,:,i)-min(Ei_mask))/range(Ei_mask) - 1; % fits range 0:+1 within mask
        E(:,:,i) = 2*(E(:,:,i)-min(Ei(:)))/range(Ei(:)) - 1; % fits range 0:+1 within mask
       % E(idx_notmask + (i-1)*numel(I)) = 0;                     % zero all values outside
    end
    
end
