%% Get spectrum
function [E,V] = spectrum_mask(I,ne,lambda_edge,lambda_node,equalize)

    if nargin<2; ne          = 8; end;    % number of eigenvectors to compute
    if nargin<3; lambda_edge = 1; end;    % factor on edge weight (0..10)
    if nargin<4; lambda_node = 1; end;    % factor on node weight (0..1)
    if nargin<5; equalize    = 1; end;    % equalize eigenmode values

    remove_unconnected = 1; % remove unconnected components (so Fielder is always 2nd vector)

    eps = 1e-6;%0.001; % small epsilon (avoid division by zero)
    th = 0.0;
    
    %% Construct adjacency matrix
    sizex = size(I,1);
    sizey = size(I,2);
    n     = sizex*sizey;

    [x,y] = ndgrid(0:(sizex-1), 0:(sizey-1)); % coordinate image
    
    % Horizontal neighbbors
    idx0x = (x+0) + (y+0)*sizex + 1; idx0x = idx0x(1:(sizex-1),1:sizey);
    idx1x = (x+1) + (y+0)*sizex + 1; idx1x = idx1x(1:(sizex-1),1:sizey);
    
    % Vertical neighbbors
    idx0y = (x+0) + (y+0)*sizex + 1; idx0y = idx0y(1:sizex,1:(sizey-1));
    idx1y = (x+0) + (y+1)*sizex + 1; idx1y = idx1y(1:sizex,1:(sizey-1));

    % Diagonal neighbbors
    idx0d1 = (x+0) + (y+0)*sizex + 1; idx0d1 = idx0d1(1:(sizex-1),1:(sizey-1));
    idx1d1 = (x+1) + (y+1)*sizex + 1; idx1d1 = idx1d1(1:(sizex-1),1:(sizey-1));

    % Diagonal neighbbors
    idx0d2 = (x+0) + (y+0)*sizex + 1; idx0d2 = idx0d2(2:sizex,1:(sizey-1));
    idx1d2 = (x-1) + (y+1)*sizex + 1; idx1d2 = idx1d2(2:sizex,1:(sizey-1));

    % Horizontal second neighbbors
    idx0x2 = [];%(x+0) + (y+0)*sizex + 1; idx0x2 = idx0x2(1:(sizex-2),1:sizey);
    idx1x2 = [];%(x+2) + (y+0)*sizex + 1; idx1x2 = idx1x2(1:(sizex-2),1:sizey);
    
    % Vertical second neighbbors
    idx0y2 = [];%(x+0) + (y+0)*sizex + 1; idx0y2 = idx0y2(1:sizex,1:(sizey-2));
    idx1y2 = [];%(x+0) + (y+2)*sizex + 1; idx1y2 = idx1y2(1:sizex,1:(sizey-2));

    % Construct neighborhood
    F(:,1) = [idx0x(:); idx0y(:); idx0d1(:); idx0d2(:); idx0x2(:); idx0y2(:)];
    F(:,2) = [idx1x(:); idx1y(:); idx1d1(:); idx1d2(:); idx1x2(:); idx1y2(:)];
 
    % Keep only nonzeros points from F
    %nz_idx = find(I~=0);
    %keep_idx = find((I(F(:,1))~=0).*(I(F(:,2))~=0));
    nz_idx = find(I>th);
    keep_idx = find((I(F(:,1))>th).*(I(F(:,2))>th));
    F = F(keep_idx,:);
    new_idx = zeros(size(I)); new_idx(nz_idx) = 1:size(nz_idx,1);
    
    % Get weights
    dist2    = (x(F(:,1))-x(F(:,2))).^2 + (y(F(:,1))-y(F(:,2))).^2;
    diff2    = ( I(F(:,1)) - I(F(:,2)) ).^2;
    maxdist2 = max(dist2(:)); if maxdist2==0; maxdist2=1; end;
    maxdiff2 = max(diff2(:)); if maxdiff2==0; maxdiff2=1; end;
    %edge_weights  = (lambda_edge*diff2/maxdiff2.*dist2/maxdist2 + (1-lambda_edge)*dist2/maxdist2 + eps).^-1;
    %s = .3;
%     th = 0;%.2; % consider only difference larger than this threshold
%     ldiff2 = diff2(diff2>th^2); % large differences (supposed to be across region boundaries (ignoring internal diffs within regions)
%     if numel(ldiff2)>0
%         s = mean(sqrt(ldiff2)) / 3;  % average of large differences (avg. of differences above our thres. (which should be avg. across region boundaries)), sigma is a third of this avg. gap between regions
%     else
%         s = max(sqrt(diff2)) / 3;
%         if s==0; s = 0.3; end;      % if homogeneous object, choose an arbitrary kernel width (all edges are 1 anyways)
%     end
    %s = max(sqrt(diff2(:)))/3;
    %s = 0.1;
    %s = std(sqrt(diff2(:)));
    %s = mean(sqrt(diff2(:)))/3 *1.5 * 3;
    s = mean(sqrt(diff2(:)))/3 *1.5;
    disp(['edge sigma: ' num2str(s)]);
    %s = .2;
    expdiff       = exp(-diff2 / (2*s^2)) ./ (dist2/maxdist2 + eps);
    maxexpdiff    = max(expdiff(:)); if maxexpdiff==0; maxexpdiff=1; end;
    edge_weights  = (lambda_edge)*(expdiff/maxexpdiff + eps) + (1-lambda_edge)*(dist2/maxdist2 + eps).^-1;
    
    %W = sparse(F(:,1),F(:,2),weights,n,n);
    n = size(nz_idx,1);
    W = sparse(new_idx(F(:,1)),new_idx(F(:,2)),edge_weights,n,n);
    W = W + W';
    
    %% Construct Laplacian matrix
    D = spdiags(sum(W)',0,speye(n)); %D = diag(sum(W));
    L = D - W;
    
    % sanity check (should not have null column)
    if numel(find((sum(D)==0)))>0;
        %warning(['Crooked Laplacian matrix, ' num2str(numel(find((sum(D)==0)))) ' columns are null (patching matrix)']);
        warning(['Crooked Laplacian matrix, removing ' num2str(numel(find((sum(D)==0)))) ' columns']);
        idx = find((sum(D)==0));
        for i=1:size(idx,2)
            %W(idx(i),idx(i)) = 1; % putting 1 in matrix (adding trivial equation)
            %L(idx(i),idx(i)) = 1;
            I(nz_idx(idx(i))) = 0;
            W(idx(i),:) = []; W(:,idx(i)) = []; % removing column
            L(idx(i),:) = []; L(:,idx(i)) = [];
            nz_idx(idx(i)) = [];
            idx(idx>idx(i)) = idx(idx>idx(i))-1;
            n = n-1;
        end
    end;

    % Node weight
    %s = 0.3;
    %node_weights = exp(-(I(nz_idx)).^2/(2*s^2));
    node_weights = 1-I(nz_idx);
    node_weights = lambda_node*(node_weights + eps) + (1-lambda_node)*ones(n,1);
    
    % Normalize
    Dinv = spdiags(sum(W)'.^-1,0,speye(n));  % Dinv = D^-1;
    N    = spdiags(node_weights,0,speye(n)); % Node weight
    G    = Dinv .* N;
    L    = G * L;            % assymetric version
%     Gsqrt= diag(diag(G).^.5); 
%     L    = Gsqrt * L * Gsqrt; % symmetric version

    %% Get Eigenvectors
    opt   = struct('disp',0, 'isreal',1, 'issym',0, 'maxit',50, 'tol',1e-5);
    %opt   = struct('disp',0, 'isreal',1, 'issym',1, 'maxit',50, 'tol',1e-10);
    try
        lastwarn('');
        [E,V,flag] = eigs(L,ne,'sm',opt); % arpack eigen decomposition
        %[E,V,flag] = eigs(L,ne,-1,opt); % arpack eigen decomposition (closest to -1, to avoid errors when computing 0)
    catch
        warning('Failed finding smallest magnitude eigenvalues, modifying matrix and removing extra null vectors');
        try
            [E,V,flag] = eigs(L,ne,-1,opt); % arpack eigen decomposition (closest to -1, to avoid errors when computing 0)
        catch
            l = 1-1e-3;                         % there might be other null vectors
            L = l*L + (1-l)*speye(n);           % try to change the matrix a little bit
            [E,V,flag] = eigs(L,ne,-1,opt); % arpack eigen decomposition (closest to -1, to avoid errors when computing 0)
        end
    end
    if flag~=0; disp('All eigenvalues did not converge'); end;
    if ~isempty(lastwarn);
        warning('Failed finding smallest magnitude eigenvalues, modifying matrix and removing extra null vectors');
        [E,V,flag] = eigs(L,ne,-1,opt); % arpack eigen decomposition (closest to -1, to avoid errors when computing 0)
    end
    V = diag(V);
    
    %% Remove smallest unconnected components
    if remove_unconnected
        idx = find(V<eps);
        if numel(idx)>1
            warning(['Removing ' num2str(numel(idx)-1) ' unconnected components']);
            for i=1:size(idx,1)
                % Check which area is smaller
                Et = abs(E(:,idx(i)));
                Et = (Et-min(Et(:)))/range(Et(:));
                area1 = sum(Et<.5);
                area2 = sum(Et>=.5);

                % Remove smaller area
                if area1<area2
                    I(nz_idx(Et<.5)) = 0;
                else
                    I(nz_idx(Et>=.5)) = 0;
                end
            end
            
            % Compute eigenvectors for largest area only
            [E,V] = spectrum_mask(I,ne,lambda_edge,lambda_node,equalize);
            return;
        end
    end
    
    %% Safety net
    V = real(V);
    E = real(E);
    
    %% Sort with eigenvalues
    [V,eidx] = sort(V);
    E = E(:,eidx);
    
    % Refill matrix with zero elements
    Et = zeros(sizex*sizey,ne);
    Et(nz_idx,:) = E; E = Et;
    
    % Reorder eigenvalues
    [V,idx] = sort(V,'ascend');
    E = E(:,idx);
    
    % Check sign in null vector
    if min(E(:,1))<0; E(:,1) = -E(:,1); end; % avoid negative values in null vector
        
    % Reshape matrix
    E = reshape(E,sizex,sizey,ne);
    
    % Scale to -1..+1
    for i=1:ne
        Ei       = E(:,:,i);
        %Ei       = 2*(E(:,:,i)-min(Ei(:)))/range(Ei(:)) - 1; % fits range -1:+1 within mask
        %m        = min(-min(Ei(:)),max(Ei(:))); if m==0; m=1; end;
        %Ei       = E(:,:,i) / m; % fits range -1:+1 within mask
        %mn       = -min(Ei(:)); % minimal eigenmode value
        %mp       =  max(Ei(:)); % maximal eigenmode value
        
        %mn = min(mn,mp); mp = mn;
        %mn       = -mean(Ei(Ei<0)); % minimal eigenmode value
        %mp       =  mean(Ei(Ei>0)); % maximal eigenmode value
        %Ei(Ei<0) = Ei(Ei<0)/mn; % fits range [-1:0] in the negative values
        %Ei(Ei>0) = Ei(Ei>0)/mp; % fits range [0:+1] in the positive values
        
        if i==1;
            
            if mean(Ei(:))<0; Ei = -Ei; end; % null vector should be positive
            Ei = 2*(Ei-0.5);                 % range [-1:+1]
            
        else
        
            if 1
                if equalize
                    n = Ei(Ei<0); n = equalizehistogram(n,5000,n~=0);
                    p = Ei(Ei>0); p = equalizehistogram(p,5000,p~=0);
                    Ei(Ei<0) = n;
                    Ei(Ei>0) = p;
                end
                mn       = -mean(Ei(Ei<0))*2; % minimal eigenmode value
                mp       =  mean(Ei(Ei>0))*2; % maximal eigenmode value
                Ei(Ei<0) = Ei(Ei<0)/mn; % fits range [-1:0] in the negative values
                Ei(Ei>0) = Ei(Ei>0)/mp; % fits range [0:+1] in the positive values
            %else
                Ei(Ei<0) = -Ei(Ei<0)/min(Ei(Ei<0)); % fits range [-1:0] in the negative values
                Ei(Ei>0) = +Ei(Ei>0)/max(Ei(Ei>0)); % fits range [0:+1] in the positive values
            end
        end
        
        E(:,:,i) = Ei;
    end

    % Zero outside the mask
    for i=2:ne
        Ei       = E(:,:,i);
        Ei(I==0) = 0; % zero outside the mask
        
        E(:,:,i) = Ei;
    end
    

end
